#### Code by Trisevgeni Papakonstantinou, Behavioural Insights Advisor, PHEBI ####

# Import packages
library(pacman)
p_load(tidyverse, readxl, janitor, stringr, plyr, dplyr, uuid, ids, data.table, quanteda, tidytext, textfeatures, tabyl, patchwork, knitr, stm)

# Make sure that data is in csv format, if excel format save the file as csv using excel
data <- read.csv("surveydata.csv")

# Keep only subset of data that is complete responses
# Contrary to qual analysis the model can't handle missing values
data <- data[data$Finished=="True",] # means: keep only rows that have a value of "True" in column "Finished"
data <- data[-1,] # means: keep all rows except (-) row "1"

# Format any variables of interest other than the free text column(s) that will be the basis of the analysis.
# Examples:
# Date variable: data$date <- format(as.Date(data$date), "%d/%m/%Y")
# Categorical variable: data$age <- as.character(data$age) -> making sure it's being read as a character 

data$age <- data$Q34
data$education <- data$Q47
data$sex <- data$Q65
data$ethnicity <- data$Q48

data$age <- as.character(data$age)
data$education <- as.character(data$education)
data$sex <- as.character(data$sex)
data$ethnicity <- as.character(data$ethnicity)

data$RecordedDate <- format(as.Date(data$RecordedDate), '%y/%m/%d')

# Keep only columns we are interested in
data <- data[,c('ResponseId', 'RecordedDate', 'age', 'education', 'sex', 'ethnicity', 'Q30', 'Q31', 'Q50')]

# Remove rows where there are NA values
data <- data[complete.cases(data),]

# Keep only rows that don't have an empty value in column 'age' etc...
# There is no need to do this for the free text columns as the empty values there are treated later
data <- data[!data$age=='',]
data <- data[!data$education=='',]
data <- data[!data$sex=='',]
data <- data[!data$ethnicity=='',]



# Make sure that the free text column are read as strings
# Example: data$text_column_name <- as.character(data$text_column_name)
data$Q30 <- as.character(data$Q30)
data$Q31 <- as.character(data$Q31)
data$Q50 <- as.character(data$Q50)

#data$Q30 <- str_replace_all(data$Q30, "???T", "'")
data$Q30 <- str_replace_all(data$Q30, "\\?\\?\\?T", "'")

# Q50 (If possible, please give us a little more information about your experience with coronavirus)

# 3 different topic models for the 3 text columns
#### STM - Q30 (What was helpful about the information on the Germ Defence website?) ####
# Create corpus named t
t <- corpus(data, text_field = "Q30")
# Match the documents with their unique IDs from the dataframe stored in column "ResponseId" here
docvars(t, "doc_id") <- data$ResponseId
# Feed the rest of the variables/columns back to t
docvars(t) <- data

# Clean text
t <- str_replace_all(t, "[']", "") # Remove ' so that the "t" from "don't" does not get read as its own word: "don't" -> "dont"
t <- tolower(t) # Turn all characters to lower case
t <- str_replace_all(t, "[^[a-z]]", " ") # Remove all characters that are not lower case alphabetical characters (e.g. numbers, punctuation)

# Create Document-Feature Matrix
d <- dfm(t, remove = stopwords("en"), # Remove stopwords (in english)
         stem = TRUE) # Use stemming to retain the meaning of words without too many particulars (e.g. instances "playing" and "player" are now both being read as "play-")
docnames(d) <- data$ResponseId # Feed the ID back so that we keep track of the unique documents

# Convert DFM to stm format 
s <- convert(d, to = "stm") 

# Delete documents that did not make it into s from data, so that they are the same length
# and we can then feed the metadata back into s
docnames_s <- s$documents
docnames_s <- names(docnames_s)
data <- data[data$ResponseId %in% docnames_s, ]


# Define metadata
# Need to subset the data like so and define columns you want to use by index! otherwise it's not going to do anything
s$meta <- data[,2:6]


# Determine optimal number of topics
set.seed(42)
K<-c(5,10,15,20,25,30,35,40)
kresult <- searchK(s$documents, s$vocab, K, data=s$meta, max.em.its = 1000)
# No objective optimal number of topics - interpret plots* and determine number according to priorities of the project and also
# factor in how much specificity is needed (less topics = less specificity)
png(file='Q30_kresult.png')
plot.searchK(kresult)
dev.off

## Stm model with month as covariate
set.seed(123)
stm_fit <- stm(documents = s$documents, vocab = s$vocab, data = s$meta, K = 25, prevalence=~RecordedDate+age+sex+education+ethnicity, init = "Spectral", max.em.its = 500)
png(file='Q30_top_topics.png')
plot(stm_fit, n = 8, labeltype = "prob", text.cex = .5)
dev.offtop
labelTopics(stm_fit)


# Stm output with weights/prevalence for each topic as column - save as dataframe
stm_df <- make.dt(stm_fit, meta = s$meta)
write.csv(stm_df, 'TQ30_Topics_Prevalence.csv')


# Print topic words
sink(file='Q30_labels.txt')
labelTopics(stm_fit)
sink(file=NULL)


### Visuals ###

# Generate quote graphs with most representative extracts for each topic
for (i in c(1:25)){
  
  thoughts <- findThoughts(stm_fit, texts = data$Q30,
                           n = 10, topics = i)$docs[[1]]
  
  png(file=paste0("Q30_quotes_topic", i, ".png"), res=100)
  plotQuote(thoughts, width = 100, text.cex = 0.5, main = paste0("Topic ", i))
  dev.off()
}




#### Repeat for Q31 ####
# Read data
data <- read.csv("Copy of Germ Defence PPI feedback survey_anonymised.csv")

# Keep only subset of data that is complete responses
# Contrary to qual analysis the model can't handle missing values
data <- data[data$Finished=="True",] 
data <- data[-1,] 

# Format any variables of interest other than the free text column(s) that will be the basis of the analysis.
data$age <- data$Q34
data$education <- data$Q47
data$sex <- data$Q65
data$ethnicity <- data$Q48


# Keep only columns we are interested in
data <- data[,c('ResponseId', 'RecordedDate', 'age', 'education', 'sex', 'ethnicity', 'Q30', 'Q31', 'Q50')]

# Remove rows where there are NA values
data <- data[complete.cases(data),]

data <- data[!data$age=='',]
data <- data[!data$education=='',]
data <- data[!data$sex=='',]
data <- data[!data$ethnicity=='',]

data$age <- as.character(data$age)
data$education <- as.character(data$education)
data$sex <- as.character(data$sex)
data$ethnicity <- as.character(data$ethnicity)

data$RecordedDate <- as.Date(data$RecordedDate, "%d/%m/%y")
data$RecordedDate <- as.character(data$RecordedDate)

# Make sure that the free text column are read as strings
data$Q30 <- as.character(data$Q30)
data$Q31 <- as.character(data$Q31)
data$Q50 <- as.character(data$Q50)


data$Q50 <- str_replace_all(data$Q50, "\\?\\?\\?T", "'") 
data$Q31 <- str_replace_all(data$Q31, "\\?\\?\\?T", "'") 



#### STM - Q31 (What did you not find helpful about the information on the Germ Defence website??) ####
# Create corpus named t
t <- corpus(data, text_field = "Q31")
# Match the documents with their unique IDs from the dataframe stored in column "ResponseId" here
docvars(t, "doc_id") <- data$ResponseId
# Feed the rest of the variables/columns back to t
docvars(t) <- data

# Clean text
t <- str_replace_all(t, "[']", "") # Remove ' so that the "t" from "don't" does not get read as its own word: "don't" -> "dont"
t <- tolower(t) # Turn all characters to lower case
t <- str_replace_all(t, "[^[a-z]]", " ") # Remove all characters that are not lower case alphabetical characters (e.g. numbers, punctuation)

# Create Document-Feature Matrix
d <- dfm(t, remove = stopwords("en"), # Remove stopwords (in english)
         stem = TRUE) # Use stemming to retain the meaning of words without too many particulars (e.g. instances "playing" and "player" are now both being read as "play-")
docnames(d) <- data$ResponseId # Feed the ID back so that we keep track of the unique documents

# Convert DFM to stm format 
s <- convert(d, to = "stm") 

# Delete documents that did not make it into s from data, so that they are the same length
# and we can then feed the metadata back into s
docnames_s <- s$documents
docnames_s <- names(docnames_s)
data <- data[data$ResponseId %in% docnames_s, ]


# Define metadata
s$meta <- data[,2:6]


# Determine optimal number of topics
set.seed(42)
K<-c(5,10,15,20,25,30,35,40)
kresult <- searchK(s$documents, s$vocab, K, data=s$meta, max.em.its = 1000)
png(file='Q31_kresult.png')
plot.searchK(kresult)
dev.off



## Stm model 
set.seed(123)
stm_fit <- stm(documents = s$documents, vocab = s$vocab, data = s$meta, K = 10, prevalence=~RecordedDate+age+sex+education+ethnicity, init = "Spectral", max.em.its = 500)
png(file='Q31_top_topicsa.png')
plot(stm_fit, n = 8, labeltype = "prob", text.cex = .6)
dev.off
labelTopics(stm_fit)


# Stm output with weights/prevalence for each topic as column - save as dataframe
stm_df <- make.dt(stm_fit, meta = s$meta)
write.csv(stm_df, 'Germ_Defence_Q31_Topics_Prevalence.csv')


# Print topic words
sink(file='Q31_labels.txt')
labelTopics(stm_fit)
sink(file=NULL)


### Visuals ###

# Generate quote graphs with most representative extracts for each topic
for (i in c(1:10)){
  
  thoughts <- findThoughts(stm_fit, texts = data$Q31,
                           n = 10, topics = i)$docs[[1]]
  print(labelTopics(stm_fit,i))
  print(thoughts)
  png(file=paste0("Q31_quotes_topic", i, ".png"), res=100)
  plotQuote(thoughts, width = 100, text.cex = 0.3, main = paste0("Topic ", i))
  dev.off()
}




